@@ -22,8 +22,16 @@
  */
 
 #include "global.h"
+#include "database.h"
 #include "display.h"
 
+/* File Extensions to use. */
+const db_extension_info_t db_extensions[] = {
+    {"info", BASENAME_META, insert_meta_data, save_meta_data, NULL },
+    {"stat", BASENAME_STAT, set_stat_data, save_stat_data, is_stat_data_changed}
+};
+const int db_extensions_size = sizeof( db_extensions ) / sizeof( db_extensions[0] );
+
 /*
  * Setup the config file keys
  */
@@ -32,12 +40,15 @@ config_keys_t config_keys[ CONFIG_KEY_COUNT ] = {
     { "Database", "Stat_Path", (void *)&config.db_paths[ BASENAME_STAT ], TYPE_STRING },
     { "Database", "Song_Path", (void *)&config.db_paths[ BASENAME_SONG ], TYPE_STRING },
     { "Database", "Readonly", (void *)&config.db_readonly, TYPE_INT },
+    { "Database", "Save_Info", (void *)&config.db_saveinfo, TYPE_INT },
+    { "Database", "Overwrite_Info", (void *)&config.db_overwriteinfo, TYPE_INT },
     { "Global", "Control_Filename", (void *)&config.input_fifo_path, TYPE_STRING },
 #ifdef DEBUG
     { "Global", "Log_Filename", (void *)&config.squash_log_path, TYPE_STRING },
 #endif
     { "Playlist", "Filename", (void *)&config.playlist_manager_playlist_path, TYPE_STRING },
-    { "Playlist", "Size", (void *)&config.playlist_manager_playlist_size, TYPE_INT }
+    { "Playlist", "Size", (void *)&config.playlist_manager_playlist_size, TYPE_INT },
+    { "Pastlist", "Size", (void *)&config.playlist_manager_pastlist_size, TYPE_INT }
 };
 
 
@@ -173,7 +184,7 @@ readonly=0
  * You must make a copy of header if you want to have a copy, you must free
  * key and value if you do not want a copy.
  */
-void parse_file( const char *file_name, void(*add_data)(void*, char*, char*, char*), void *data ) {
+bool parse_file( const char *file_name, void(*add_data)(void*, char*, char*, char*), void *data ) {
     /* Position Variables */
     char *cur_pos = NULL;
     char *cur_start = NULL;
@@ -198,22 +209,26 @@ void parse_file( const char *file_name, void(*add_data)(void*, char*, char*, cha
     FILE *file_desc;
     struct stat file_info;
     char *file_data = NULL;
+    bool exit = TRUE;
 
     /* Open the file */
     if( (file_desc = fopen(file_name, "r")) == NULL ) {
         /* Error opening the file */
+        exit = FALSE;
         goto cleanup;
     }
 
     /* Get information on the file */
     if( fstat(fileno(file_desc), &file_info) ) {
         /* Error loading file information */
+        exit = FALSE;
         goto cleanup;
     }
 
     /* Map the file into memory */
     if( (file_data = (char *)mmap( (void *)NULL, file_info.st_size, PROT_READ, MAP_SHARED, fileno(file_desc), 0)) == MAP_FAILED ) {
         /* Error mapping the file */
+        exit = FALSE;
         goto cleanup;
     }
 
@@ -379,6 +394,8 @@ cleanup:
         fclose( file_desc );
         file_desc = NULL;
     }
+
+    return exit;
 }
 
 /* Used by parse_file() to set config values read */
@@ -439,6 +456,8 @@ void init_config( void ) {
     config.db_paths[ BASENAME_META ] = NULL;
     config.db_paths[ BASENAME_STAT ] = NULL;
     config.db_readonly = 0;
+    config.db_saveinfo = 1;
+    config.db_overwriteinfo = 0;
 
     /* Control Options */
     config.input_fifo_path = strdup("~/.squash_control");
@@ -446,6 +465,7 @@ void init_config( void ) {
     /* Playlist Manager Options */
     config.playlist_manager_playlist_path = strdup("~/.squash_playlist");
     config.playlist_manager_playlist_size = 32;
+    config.playlist_manager_pastlist_size = 32;
 
     /* Debug Options */
 #ifdef DEBUG
@@ -515,3 +535,66 @@ char *copy_string( const char *start, const char *end ) {
     return new_string;
 }
 
+/*
+ * This combines the basename of a song with the song's filename
+ */
+char *build_fullfilename( song_info_t *song, enum basename_type_e type ) {
+    int length;
+    int ext;
+    char *filename;
+
+    if( song == NULL ) {
+        return NULL;
+    }
+
+    length = strlen(song->basename[type]) + strlen(song->filename) + 2;
+    if( type != BASENAME_SONG ) {
+        /* TODO: fix this with a table */
+        if( type == BASENAME_META ) {
+            ext = TYPE_META;
+        } else {
+            ext = TYPE_STAT;
+        }
+        length += strlen(db_extensions[ext].extension) + 1;
+
+        squash_malloc( filename, length);
+        snprintf( filename, length, "%s/%s.%s", song->basename[type], song->filename, db_extensions[ext].extension );
+    } else {
+        squash_malloc( filename, length);
+        snprintf( filename, length, "%s/%s", song->basename[type], song->filename );
+    }
+
+    return filename;
+}
+
+/*
+ * creates a directory tree.  note that dir is temporarily modified, but it's
+ * value is restored by the end of the routine.
+ */
+void create_path( char *dir ) {
+    char *cur;
+    int result;
+
+    cur = dir;
+    while( *cur != '\0' ) {
+        if( *cur == '/' ) {
+            if( cur - dir > 0 ) {
+                *cur = '\0';
+
+                result = mkdir( dir, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP|S_IROTH|S_IWOTH|S_IXOTH );
+                if( result != 0 && errno != EEXIST ) {
+                    squash_error("Could not create info file directory '%s' (%d)", dir, errno );
+                }
+                *cur = '/';
+            }
+        }
+        cur++;
+    }
+
+    result = mkdir( dir, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP|S_IROTH|S_IWOTH|S_IXOTH );
+    if( result != 0 && errno != EEXIST ) {
+        squash_error("Could not create info file directory '%s' (%d)", dir, errno );
+    }
+}
+
+
